package com.yfd.platform.modules.algorithm.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.data.Pictures;
import com.deepoove.poi.plugin.table.HackLoopTableRenderPolicy;
import com.deepoove.poi.util.PoitlIOUtils;
import com.yfd.platform.config.HttpServerConfig;
import com.yfd.platform.config.ResponseResult;
import com.yfd.platform.modules.algorithm.domain.AlgorithmArrange;
import com.yfd.platform.modules.algorithm.service.IAlgorithmArrangeDeviceService;
import com.yfd.platform.modules.algorithm.service.IAlgorithmArrangeService;
import com.yfd.platform.modules.basedata.domain.Substation;
import com.yfd.platform.system.service.ISysDictionaryItemsService;
import com.yfd.platform.utils.FileUtil;
import com.yfd.platform.utils.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.ClassUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLDecoder;
import java.nio.file.Files;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zhengsl
 * @since 2025-06-06
 */
@RestController
@RequestMapping("/algorithm/algorithm-arrange")
@Api(value = "AlgorithmArrangeController", tags = "算法布点模块")
@Slf4j
public class AlgorithmArrangeController {

    @Resource
    private IAlgorithmArrangeService algorithmArrangeService;
    @Resource
    private IAlgorithmArrangeDeviceService algorithmArrangeDeviceService;
    @Resource
    private ISysDictionaryItemsService sysDictionaryItemsService;
    @Resource
    private HttpServerConfig httpServerConfig;

    @GetMapping("/getAlgorithmArrangePage")
    @ApiOperation("查询算法布点")
    public ResponseResult getAlgorithmArrangePage(Page<AlgorithmArrange> page, String stationId, String stationCode,
                                                  String arrangeName,
                                                  String businessType) {
        LambdaQueryWrapper<AlgorithmArrange> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StrUtil.isNotBlank(stationId), AlgorithmArrange::getStationId, stationId);
        queryWrapper.eq(StrUtil.isNotBlank(stationCode), AlgorithmArrange::getStationCode, stationCode);
        queryWrapper.like(StrUtil.isNotBlank(arrangeName), AlgorithmArrange::getArrangeName, arrangeName);
        queryWrapper.eq(StrUtil.isNotBlank(businessType), AlgorithmArrange::getBusinessType, businessType);
        queryWrapper.eq(AlgorithmArrange::getDatastatus, "1");
        Page<AlgorithmArrange> arrangePage = algorithmArrangeService.page(page, queryWrapper);
        return ResponseResult.successData(arrangePage);
    }

    @GetMapping("/getAlgorithmArrangeById")
    @ApiOperation("根据Id查询算法布点")
    public ResponseResult getAlgorithmArrangeById(String id) {
        AlgorithmArrange algorithmArrange = algorithmArrangeService.getById(id);
        return ResponseResult.successData(algorithmArrange);
    }

    @PostMapping("/saveAlgorithmArrange")
    @ApiOperation("新增算法布点")
    public ResponseResult saveAlgorithmArrange(AlgorithmArrange algorithmArrange, MultipartFile file) {

        // 判断是否存在相同的方案
        LambdaQueryWrapper<AlgorithmArrange> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AlgorithmArrange::getArrangeName, algorithmArrange.getArrangeName());
        if (algorithmArrangeService.getOne(queryWrapper) != null) {
            return ResponseResult.error("方案名称已存在");
        }
        algorithmArrange.setLastmodifier("admin");
        algorithmArrange.setLastmodifydate(LocalDateTime.now());
        algorithmArrange.setDatastatus("1");
        String id = IdUtil.fastSimpleUUID();
        algorithmArrange.setId(id);
        // 文件上传逻辑
        if (file != null && !file.isEmpty()) {
            try {
                // 上传文件
                String fileUrl = algorithmArrangeService.uploadImage(file);
                algorithmArrange.setImageUrl(fileUrl);
                algorithmArrange.setIsImageFlag("1");
            } catch (Exception e) {
                return ResponseResult.error("文件上传失败：" + e.getMessage());
            }
        }
        boolean save = algorithmArrangeService.save(algorithmArrange);
        if (!save) {
            return ResponseResult.error();
        }
        return ResponseResult.successData(algorithmArrange);
    }

    @PostMapping("/saveArrangeImage")
    @ApiOperation("保存布点图片")
    public ResponseResult saveArrangeImage(String id, MultipartFile file) {
        // 文件上传逻辑
        if (file != null && !file.isEmpty()) {
            try {
                // 上传文件
                String fileUrl = algorithmArrangeService.uploadImage(file);
                LambdaUpdateWrapper<AlgorithmArrange> updateWrapper = new LambdaUpdateWrapper<>();
                updateWrapper.eq(AlgorithmArrange::getId, id).set(AlgorithmArrange::getArrangeImageUrl, fileUrl);
                algorithmArrangeService.update(updateWrapper);
            } catch (Exception e) {
                return ResponseResult.error("文件上传失败：" + e.getMessage());
            }
        }
        return ResponseResult.success();
    }

    @PostMapping("/updateAlgorithmArrange")
    @ApiOperation("修改算法布点")
    public ResponseResult updateAlgorithmArrange(AlgorithmArrange algorithmArrange, MultipartFile file) {
        // 判断是否存在相同的方案
        LambdaQueryWrapper<AlgorithmArrange> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.ne(AlgorithmArrange::getId, algorithmArrange.getId()).eq(AlgorithmArrange::getArrangeName,
                algorithmArrange.getArrangeName());
        if (algorithmArrangeService.getOne(queryWrapper) != null) {
            return ResponseResult.error("方案名称已存在");
        }
        algorithmArrange.setLastmodifier(SecurityUtils.getCurrentUsername());
        algorithmArrange.setLastmodifydate(LocalDateTime.now());
        // 文件上传逻辑
        if (file != null && !file.isEmpty()) {
            try {
                // 调用服务层方法
                String fileUrl = algorithmArrangeService.uploadImage(file);
                algorithmArrange.setImageUrl(fileUrl);
                algorithmArrange.setIsImageFlag("1");
            } catch (Exception e) {
                return ResponseResult.error("文件上传失败：" + e.getMessage());
            }
        }
        boolean save = algorithmArrangeService.updateById(algorithmArrange);
        if (!save) {
            return ResponseResult.error();
        }
        return ResponseResult.success();
    }

    @PostMapping("/deleteAlgorithmArrange")
    @ApiOperation("删除布点数据")
    public ResponseResult deleteAlgorithmArrange(String id) {
        boolean b = algorithmArrangeService.removeById(id);
        if (b) {
            return ResponseResult.success();
        } else {
            return ResponseResult.error();
        }
    }

    @PostMapping("/deleteAlgorithmArrangeByIds")
    @ApiOperation("删除布点数据")
    public ResponseResult deleteAlgorithmArrangeByIds(@RequestBody List<String> ids) {
        boolean b = algorithmArrangeService.removeByIds(ids);
        if (b) {
            return ResponseResult.success();
        } else {
            return ResponseResult.error();
        }
    }

    @PostMapping("/callAlgorithmArrange")
    @ApiOperation("调用算法布点API")
    public ResponseResult callAlgorithmArrange(String id) {
        AlgorithmArrange algorithmArrange = algorithmArrangeService.callAlgorithmArrange(id);
        return ResponseResult.successData(algorithmArrange);
    }

    @GetMapping("/exportArrangeById")
    @ApiOperation("导出布点报告")
    public void exportArrangeById(String id, HttpServletResponse response) throws IOException {
        Map<String, Object> map = new HashMap<>();
        AlgorithmArrange algorithmArrange = algorithmArrangeService.getById(id);
        map.put("arrangeName", algorithmArrange.getArrangeName());
        map.put("stationName", algorithmArrange.getStationName());
        Map<String, Object> businessTypeMap = sysDictionaryItemsService.getDeviceMapByType("businessType");
        Map<String, Object> stationTypeMap = sysDictionaryItemsService.getDeviceMapByType("stationType");
        JSONObject businessTypeJSON= JSONUtil.parseObj(businessTypeMap.get("resultMap"));
        JSONObject stationTypeJSON= JSONUtil.parseObj(stationTypeMap.get("resultMap"));
        String businessType = StrUtil.isNotBlank(algorithmArrange.getBusinessType()) ? businessTypeJSON.getStr(algorithmArrange.getBusinessType()) : "";
        String stationType = StrUtil.isNotBlank(algorithmArrange.getStationType()) ? stationTypeJSON.getStr(algorithmArrange.getStationType()) : "";
        map.put("stationType", stationType);
        map.put("businessType",businessType);
        map.put("voltLevel", algorithmArrange.getVoltLevel());
        map.put("status", "完成");
        map.put("date", DateUtil.now());
        List<Map<String, Object>> arrangeDeviceInfo = algorithmArrangeDeviceService.getArrangeDeviceInfo(id);
        arrangeDeviceInfo.forEach(a -> {
            // 图片流
            a.put("index", arrangeDeviceInfo.indexOf(a) + 1);
        });
        try {

            String arrangeImageUrl = algorithmArrange.getArrangeImageUrl();
            if (StrUtil.isNotBlank(arrangeImageUrl)) {
                map.put("img",
                        Pictures.ofStream(Files.newInputStream(new File(httpServerConfig.getPlanFilePath() + arrangeImageUrl).toPath()), FileUtil.getPictureType(arrangeImageUrl)).fitSize().create());
            }

        } catch (Exception e) {
            log.error(e.getMessage());
        }
        List<Map<String, Object>> demo = getMapData();
        //创建一个列表的规则
        HackLoopTableRenderPolicy policy = new HackLoopTableRenderPolicy();
        //设置列表配置，如果有多个列表时需加.bind("list1", policy) 新列表配置即可
        Configure config = Configure.builder().bind("list", policy).build();
        if (!arrangeDeviceInfo.isEmpty()) {
            map.put("list", arrangeDeviceInfo);
        } else {
            map.put("list", demo);
        }
        // 加载模板渲染数据
        String path = Objects.requireNonNull(Objects.requireNonNull(ClassUtils.getDefaultClassLoader()).getResource(
                "")).getPath() + "templates" + File.separator + "布点报告模板.docx";
        XWPFTemplate compile = XWPFTemplate.compile(path, config);
        XWPFTemplate template = compile.render(map);
//        template.write(new FileOutputStream("E:\\巡视报告模板.docx"));
        response.setContentType("application/octet-stream");
        response.setHeader("Content-disposition", "attachment;filename=reportTemp.docx");

        OutputStream out = response.getOutputStream();
        BufferedOutputStream bos = new BufferedOutputStream(out);
        template.write(bos);
        bos.flush();
        out.flush();
        PoitlIOUtils.closeQuietlyMulti(template, bos, out);
    }

    private List<Map<String, Object>> getMapData() {
        List<Map<String, Object>> demo = new ArrayList<>();
        Map<String, Object> map = new HashMap<>();
        map.put("index", "");
        map.put("areaName", "");
        map.put("1", "");
        map.put("2", "");
        map.put("3", "");
        map.put("4", "");
        demo.add(map);
        return demo;
    }

}
